@@ -6,11 +6,11 @@ WWW::Mechanize - Handy web browsing in a Perl object
 
 =head1 VERSION
 
-Version 1.62
+Version 1.64
 
 =cut
 
-our $VERSION = '1.62';
+our $VERSION = '1.64';
 
 =head1 SYNOPSIS
 
@@ -280,7 +280,7 @@ sub new {
     $self->{proxy} = {} unless defined $self->{proxy};
     push( @{$self->requests_redirectable}, 'POST' );
 
-    $self->_reset_page;
+    $self->_reset_page();
 
     return $self;
 }
@@ -559,7 +559,6 @@ sub status {        my $self = shift; return $self->{status}; }
 sub ct {            my $self = shift; return $self->{ct}; }
 sub content_type {  my $self = shift; return $self->{ct}; }
 sub base {          my $self = shift; return $self->{base}; }
-sub current_form {  my $self = shift; return $self->{form}; }
 sub is_html {       my $self = shift; return defined $self->ct && ($self->ct eq 'text/html'); }
 
 =head2 $mech->title()
@@ -571,12 +570,16 @@ L<HTML::HeadParser>.  Returns undef if the content is not HTML.
 
 sub title {
     my $self = shift;
+
     return unless $self->is_html;
 
-    require HTML::HeadParser;
-    my $p = HTML::HeadParser->new;
-    $p->parse($self->content);
-    return $p->header('Title');
+    if ( not defined $self->{title} ) {
+        require HTML::HeadParser;
+        my $p = HTML::HeadParser->new;
+        $p->parse($self->content);
+        $self->{title} = $p->header('Title');
+    }
+    return $self->{title};
 }
 
 =head1 CONTENT-HANDLING METHODS
@@ -686,7 +689,7 @@ links.  In scalar context, returns an array reference of all links.
 sub links {
     my $self = shift;
 
-    $self->_extract_links() unless $self->{_extracted_links};
+    $self->_extract_links() unless $self->{links};
 
     return @{$self->{links}} if wantarray;
     return $self->{links};
@@ -1055,7 +1058,7 @@ images.  In scalar context, returns an array reference of all images.
 sub images {
     my $self = shift;
 
-    $self->_extract_images() unless $self->{_extracted_images};
+    $self->_extract_images() unless $self->{images};
 
     return @{$self->{images}} if wantarray;
     return $self->{images};
@@ -1213,10 +1216,22 @@ context, returns an array reference of all forms.
 
 sub forms {
     my $self = shift;
+
+    $self->_extract_forms() unless $self->{forms};
+
     return @{$self->{forms}} if wantarray;
     return $self->{forms};
 }
 
+sub current_form {
+    my $self = shift;
+
+    if ( !$self->{current_form} ) {
+        $self->form_number(1);
+    }
+
+    return $self->{current_form};
+}
 
 =head2 $mech->form_number($number)
 
@@ -1237,9 +1252,10 @@ sub form_number {
     my ($self, $form) = @_;
     # XXX Should we die if no $form is defined? Same question for form_name()
 
-    if ($self->{forms}->[$form-1]) {
-        $self->{form} = $self->{forms}->[$form-1];
-        return $self->{form};
+    my $forms = $self->forms;
+    if ( $forms->[$form-1] ) {
+        $self->{current_form} = $forms->[$form-1];
+        return $self->{current_form};
     }
 
     return;
@@ -1264,10 +1280,13 @@ sub form_name {
 
     my $temp;
     my @matches = grep {defined($temp = $_->attr('name')) and ($temp eq $form) } $self->forms;
-    if ( my $nmatches = @matches ) {
-        $self->warn( "There are $nmatches forms named $form.  The first one was used." )
-            if $nmatches > 1;
-        return $self->{form} = $matches[0];
+
+    my $nmatches = @matches;
+    if ( $nmatches > 0 ) {
+        if ( $nmatches > 1 ) {
+            $self->warn( "There are $nmatches forms named $form.  The first one was used." )
+        }
+        return $self->{current_form} = $matches[0];
     }
 
     return;
@@ -1294,7 +1313,7 @@ sub form_id {
     if ( @matches ) {
         $self->warn( 'There are ', scalar @matches, " forms with ID $formid.  The first one was used." )
             if @matches > 1;
-        return $self->{form} = $matches[0];
+        return $self->{current_form} = $matches[0];
     }
     else {
         $self->warn( qq{ There is no form with ID "$formid"} );
@@ -1331,10 +1350,12 @@ sub form_with_fields {
         push @matches, $form;
     }
 
-    if ( my $nmatches = @matches ) {
-        $self->warn( "There are $nmatches forms with the named fields.  The first one was used." )
-            if $nmatches > 1;
-        return $self->{form} = $matches[0];
+    my $nmatches = @matches;
+    if ( $nmatches > 0 ) {
+        if ( $nmatches > 1 ) {
+            $self->warn( "There are $nmatches forms with the named fields.  The first one was used." )
+        }
+        return $self->{current_form} = $matches[0];
     }
     else {
         $self->warn( qq{There is no form with the requested fields} );
@@ -1364,7 +1385,7 @@ sub field {
     my ($self, $name, $value, $number) = @_;
     $number ||= 1;
 
-    my $form = $self->{form};
+    my $form = $self->current_form();
     if ($number > 1) {
         $form->find_input($name, undef, $number)->value($value);
     }
@@ -1404,7 +1425,7 @@ false and calls C<< $self>warn() >> with an error message.
 sub select {
     my ($self, $name, $value) = @_;
 
-    my $form = $self->{form};
+    my $form = $self->current_form();
 
     my $input = $form->find_input($name);
     if (!$input) {
@@ -1647,7 +1668,7 @@ sub value {
     my $name = shift;
     my $number = shift || 1;
 
-    my $form = $self->{form};
+    my $form = $self->current_form;
     if ( $number > 1 ) {
         return $form->find_input( $name, undef, $number )->value();
     }
@@ -1673,7 +1694,7 @@ Returns an L<HTTP::Response> object.
 sub click {
     my ($self, $button, $x, $y) = @_;
     for ($x, $y) { $_ = 1 unless defined; }
-    my $request = $self->{form}->click($button, $x, $y);
+    my $request = $self->current_form->click($button, $x, $y);
     return $self->request( $request );
 }
 
@@ -1732,7 +1753,7 @@ sub click_button {
         $_ = 1 unless defined;
     }
 
-    my $form = $self->{form} or $self->die( 'click_button: No form has been selected' );
+    my $form = $self->current_form or $self->die( 'click_button: No form has been selected' );
 
     my $request;
     if ( $args{name} ) {
@@ -1774,7 +1795,7 @@ longer so.
 sub submit {
     my $self = shift;
 
-    my $request = $self->{form}->make_request;
+    my $request = $self->current_form->make_request;
     return $self->request( $request );
 }
 
@@ -2232,17 +2253,7 @@ sub update_html {
     $self->{ct} = 'text/html';
     $self->{content} = $html;
 
-    $self->{forms} = [ HTML::Form->parse($html, $self->base) ];
-    for my $form (@{ $self->{forms} }) {
-        for my $input ($form->inputs) {
-             if ($input->type eq 'file') {
-                 $input->value( undef );
-             }
-        }
-    }
-    $self->{form}  = $self->{forms}->[0];
-    $self->{_extracted_links} = 0;
-    $self->{_extracted_images} = 0;
+    $self->_reset_page();
 
     return;
 }
@@ -2456,13 +2467,11 @@ Resets the internal fields that track page parsed stuff.
 sub _reset_page {
     my $self = shift;
 
-    $self->{_extracted_links}  = 0;
-    $self->{_extracted_images} = 0;
-    $self->{links}             = [];
-    $self->{images}            = [];
-    $self->{forms}             = [];
-
-    delete $self->{form};
+    $self->{links}        = undef;
+    $self->{images}       = undef;
+    $self->{forms}        = undef;
+    $self->{current_form} = undef;
+    $self->{title}        = undef;
 
     return;
 }
@@ -2496,8 +2505,6 @@ sub _extract_links {
         } # while
     }
 
-    $self->{_extracted_links} = 1;
-
     return;
 }
 
@@ -2520,8 +2527,6 @@ sub _extract_images {
         } # while
     }
 
-    $self->{_extracted_images} = 1;
-
     return;
 }
 
@@ -2606,6 +2611,23 @@ sub _link_from_token {
         });
 } # _link_from_token
 
+
+sub _extract_forms {
+    my $self = shift;
+
+    my @forms = HTML::Form->parse( $self->content, $self->base );
+    $self->{forms} = \@forms;
+    for my $form ( @forms ) {
+        for my $input ($form->inputs) {
+             if ($input->type eq 'file') {
+                 $input->value( undef );
+             }
+        }
+    }
+
+    return;
+}
+
 =head2 $mech->_push_page_stack()
 
 The agent keeps a stack of visited pages, which it can pop when it needs
@@ -2852,6 +2874,7 @@ Just like Mech, but using Microsoft Internet Explorer to do the work.
 Thanks to the numerous people who have helped out on WWW::Mechanize in
 one way or another, including
 Kirrily Robert for the original C<WWW::Automate>,
+Ansgar Burchardt,
 Gisle Aas,
 Jeremy Ary,
 Hilary Holz,
